package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.base.entity.BasePriceStrategyEnt;
import pps.core.base.entity.BasePriceStrategyView;
import pps.core.base.mapper.BasePriceStrategyMapper;
import pps.core.base.mapper.BasePriceStrategyViewMapper;
import pps.core.base.service.data.base_price_strategy.*;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.List;

@XService
public class BasePriceStrategyService {
    @XApiAnonymous
    @XApiPost
    public XServiceResult createBasePriceStrategy(XContext context, CreateBasePriceStrategyInput input) {
        BasePriceStrategyMapper mapper = context.getBean(BasePriceStrategyMapper.class);
        BasePriceStrategyEnt entity = new BasePriceStrategyEnt();
        XCopyUtils.copyObject(input, entity);
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult updateBasePriceStrategy(XContext context, UpdateBasePriceStrategyInput input) {
        BasePriceStrategyMapper mapper = context.getBean(BasePriceStrategyMapper.class);
        QueryWrapper<BasePriceStrategyEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasePriceStrategyEnt::getId, input.getId());
        BasePriceStrategyEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteBasePriceStrategy(XContext context, DeleteBasePriceStrategyInput input) {
        BasePriceStrategyMapper mapper = context.getBean(BasePriceStrategyMapper.class);
        QueryWrapper<BasePriceStrategyEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasePriceStrategyEnt::getId, input.getId());
        BasePriceStrategyEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetBasePriceStrategyOutput> getBasePriceStrategy(XContext context, GetBasePriceStrategyInput input) {
        BasePriceStrategyMapper mapper = context.getBean(BasePriceStrategyMapper.class);
        QueryWrapper<BasePriceStrategyEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasePriceStrategyEnt::getId, input.getId());
        BasePriceStrategyEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBasePriceStrategyOutput output = new GetBasePriceStrategyOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBasePriceStrategyOutput> queryBasePriceStrategy(XContext context, QueryBasePriceStrategyInput input) {
        BasePriceStrategyMapper mapper = context.getBean(BasePriceStrategyMapper.class);
        QueryWrapper<BasePriceStrategyEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(BasePriceStrategyEnt::getId, input.getId())
                .eq(BasePriceStrategyEnt::getIsDeleted, input.getIsDeleted())
                .like(BasePriceStrategyEnt::getCreateById, input.getCreateById())
                .like(BasePriceStrategyEnt::getCreateByName, input.getCreateByName())
                .eq(BasePriceStrategyEnt::getCreateTime, input.getCreateTime())
                .like(BasePriceStrategyEnt::getModifyById, input.getModifyById())
                .like(BasePriceStrategyEnt::getModifyByName, input.getModifyByName())
                .eq(BasePriceStrategyEnt::getModifyTime, input.getModifyTime())
                .like(BasePriceStrategyEnt::getPolicyName, input.getPolicyName())
                .like(BasePriceStrategyEnt::getAreaCode, input.getAreaCode())
                .eq(BasePriceStrategyEnt::getStrategyYear, input.getStrategyYear())
                .eq(BasePriceStrategyEnt::getIsFixedPrice, input.getIsFixedPrice())
                .eq(BasePriceStrategyEnt::getFixedElectricityPrice, input.getFixedElectricityPrice());
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBasePriceStrategyOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBasePriceStrategyOutput> dynamicQueryBasePriceStrategy(XContext context, CustomQueryInput input) {
        BasePriceStrategyMapper mapper = context.getBean(BasePriceStrategyMapper.class);
        return XMapperHelper.query(mapper, input, BasePriceStrategyEnt.class, QueryBasePriceStrategyOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetBasePriceStrategyViewOutput> getBasePriceStrategyView(XContext context, GetBasePriceStrategyViewInput input) {
        BasePriceStrategyViewMapper mapper = context.getBean(BasePriceStrategyViewMapper.class);
        BasePriceStrategyView record = new BasePriceStrategyView();
        XCopyUtils.copyObject(input, record);
        BasePriceStrategyView view = mapper.selectOne(record);
        if (view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBasePriceStrategyViewOutput output = new GetBasePriceStrategyViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBasePriceStrategyViewOutput> queryBasePriceStrategyView(XContext context, QueryBasePriceStrategyViewInput input) {
        BasePriceStrategyViewMapper mapper = context.getBean(BasePriceStrategyViewMapper.class);
        BasePriceStrategyView record = new BasePriceStrategyView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BasePriceStrategyView> list = mapper.selectList(record);
        PageInfo<BasePriceStrategyView> pageInfo = new PageInfo<>(list);
        List<QueryBasePriceStrategyViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBasePriceStrategyViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

}
