package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.base.entity.BaseWellheadEnt;
import pps.core.base.entity.BaseWellheadView;
import pps.core.base.mapper.BaseWellheadMapper;
import pps.core.base.mapper.BaseWellheadViewMapper;
import pps.core.base.service.data.base_wellhead.*;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.List;

@XService
public class BaseWellheadService {
    @XApiAnonymous
    @XApiPost
    public XServiceResult createBaseWellhead(XContext context, CreateBaseWellheadInput input) {
        BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
        BaseWellheadEnt entity = new BaseWellheadEnt();
        XCopyUtils.copyObject(input, entity);
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult updateBaseWellhead(XContext context, UpdateBaseWellheadInput input) {
        BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
        QueryWrapper<BaseWellheadEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseWellheadEnt::getId, input.getId());
        BaseWellheadEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteBaseWellhead(XContext context, DeleteBaseWellheadInput input) {
        BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
        QueryWrapper<BaseWellheadEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseWellheadEnt::getId, input.getId());
        BaseWellheadEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetBaseWellheadOutput> getBaseWellhead(XContext context, GetBaseWellheadInput input) {
        BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
        QueryWrapper<BaseWellheadEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseWellheadEnt::getId, input.getId());
        BaseWellheadEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseWellheadOutput output = new GetBaseWellheadOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBaseWellheadOutput> queryBaseWellhead(XContext context, QueryBaseWellheadInput input) {
        BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
        QueryWrapper<BaseWellheadEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(BaseWellheadEnt::getId, input.getId())
                .eq(BaseWellheadEnt::getIsDeleted, input.getIsDeleted())
                .like(BaseWellheadEnt::getCreateById, input.getCreateById())
                .like(BaseWellheadEnt::getCreateByName, input.getCreateByName())
                .eq(BaseWellheadEnt::getCreateTime, input.getCreateTime())
                .like(BaseWellheadEnt::getModifyById, input.getModifyById())
                .like(BaseWellheadEnt::getModifyByName, input.getModifyByName())
                .eq(BaseWellheadEnt::getModifyTime, input.getModifyTime())
                .like(BaseWellheadEnt::getOuId, input.getOuId())
                .like(BaseWellheadEnt::getWellNumber, input.getWellNumber())
                .like(BaseWellheadEnt::getRunTypeKey, input.getRunTypeKey())
                .eq(BaseWellheadEnt::getServiceRating, input.getServiceRating());
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBaseWellheadOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBaseWellheadOutput> dynamicQueryBaseWellhead(XContext context, CustomQueryInput input) {
        BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
        return XMapperHelper.query(mapper, input, BaseWellheadEnt.class, QueryBaseWellheadOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetBaseWellheadViewOutput> getBaseWellheadView(XContext context, GetBaseWellheadViewInput input) {
        BaseWellheadViewMapper mapper = context.getBean(BaseWellheadViewMapper.class);
        BaseWellheadView record = new BaseWellheadView();
        XCopyUtils.copyObject(input, record);
        BaseWellheadView view = mapper.selectOne(record);
        if (view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseWellheadViewOutput output = new GetBaseWellheadViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBaseWellheadViewOutput> queryBaseWellheadView(XContext context, QueryBaseWellheadViewInput input) {
        BaseWellheadViewMapper mapper = context.getBean(BaseWellheadViewMapper.class);
        BaseWellheadView record = new BaseWellheadView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BaseWellheadView> list = mapper.selectList(record);
        PageInfo<BaseWellheadView> pageInfo = new PageInfo<>(list);
        List<QueryBaseWellheadViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseWellheadViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

}
